@@ -132,17 +132,21 @@ impl<XMSG> Component<Msg, XMSG> for Editor<XMSG> {
                 let was_cleared = self.last_char_count == Some(0);
 
                 if char_count == 1 && (was_cleared || char_count_decreased) {
+                    log::trace!("in char_count == 1..");
                     let c = input.chars().next().expect("must be only 1 chr");
                     self.composed_key = Some(c);
                     if c == '\n' {
+                        log::trace!("A new_line is pressed");
                         self.command_break_line();
                     } else {
+                        log::trace!("inserting it as a char: {:?}", c);
                         self.command_insert_char(c);
                     }
+                    log::trace!("clearing textarea");
                     self.clear_hidden_textarea();
                 }
                 self.last_char_count = Some(char_count);
-
+                log::trace!("extern messages");
                 let extern_msgs = self.emit_on_change_listeners();
                 Effects::with_external(extern_msgs).measure()
             }
@@ -181,6 +185,10 @@ impl<XMSG> Component<Msg, XMSG> for Editor<XMSG> {
                             let extern_msgs = self.emit_on_change_listeners();
                             Effects::with_external(extern_msgs).measure()
                         }
+                        'a' if is_ctrl => {
+                            self.command_select_all();
+                            Effects::none()
+                        }
                         _ => {
                             self.command_insert_char(c);
                             self.clear_hidden_textarea();
@@ -372,8 +380,62 @@ impl<XMSG> Editor<XMSG> {
         Effects::with_external(extern_msgs).measure()
     }
 
+    fn command_delete_back(&mut self) -> Effects<Msg, XMSG> {
+        let ch = self.text_buffer.command_delete_back();
+        self.text_buffer.rehighlight();
+        self.recorded.delete(ch);
+        let extern_msgs = self.emit_on_change_listeners();
+        Effects::with_external(extern_msgs).measure()
+    }
+
+    fn command_delete_forward(&mut self) -> Effects<Msg, XMSG> {
+        let ch = self.text_buffer.command_delete_forward();
+        self.recorded.delete_forward(ch);
+        let extern_msgs = self.emit_on_change_listeners();
+        Effects::with_external(extern_msgs).measure()
+    }
+
+    fn command_move_up(&mut self) {
+        self.text_buffer.move_up();
+        self.recorded.move_cursor(0, -1);
+    }
+
+    fn command_move_down(&mut self) {
+        self.text_buffer.move_down();
+        self.recorded.move_cursor(0, 1);
+    }
+
+    fn command_move_left(&mut self) {
+        self.text_buffer.move_left();
+        self.recorded.move_cursor(-1, 0);
+    }
+
+    #[allow(unused)]
+    fn command_move_left_start(&mut self) {
+        let pos = self.text_buffer.get_position();
+        self.text_buffer.move_left_start();
+        let line_width = self.text_buffer.line_width(pos.y).unwrap_or(0) as i32;
+        self.recorded.move_cursor(-line_width, 0);
+    }
+
+    fn command_move_right(&mut self) {
+        self.text_buffer.move_right();
+        self.recorded.move_cursor(1, 0);
+    }
+
     fn command_break_line(&mut self) -> Effects<Msg, XMSG> {
-        self.text_buffer.command_break_line();
+        let pos = self.text_buffer.get_position();
+        self.text_buffer.command_break_line(pos.x, pos.y);
+        self.recorded.break_line(pos.x, pos.y);
+        self.text_buffer.rehighlight();
+        let extern_msgs = self.emit_on_change_listeners();
+        Effects::with_external(extern_msgs).measure()
+    }
+
+    #[allow(unused)]
+    fn command_join_line(&mut self) -> Effects<Msg, XMSG> {
+        let pos = self.text_buffer.get_position();
+        self.text_buffer.command_join_line(pos.x, pos.y);
         self.text_buffer.rehighlight();
         let extern_msgs = self.emit_on_change_listeners();
         Effects::with_external(extern_msgs).measure()
@@ -387,8 +449,12 @@ impl<XMSG> Editor<XMSG> {
     }
 
     pub fn command_set_position(&mut self, cursor_x: i32, cursor_y: i32) {
+        let pos = self.text_buffer.get_position();
         self.text_buffer
             .set_position(cursor_x as usize, cursor_y as usize);
+        let dist_x = cursor_x - pos.x as i32;
+        let dist_y = cursor_y - pos.y as i32;
+        self.recorded.move_cursor(dist_x, dist_y);
     }
 
     fn command_set_selection(&mut self, start: Point2<i32>, end: Point2<i32>) {
@@ -397,6 +463,10 @@ impl<XMSG> Editor<XMSG> {
         self.text_buffer.set_selection(start, end);
     }
 
+    fn command_select_all(&mut self) {
+        self.text_buffer.select_all();
+    }
+
     /// calls on 2 ways to copy
     /// either 1 should work
     /// returns true if it succeded
@@ -429,14 +499,17 @@ impl<XMSG> Editor<XMSG> {
     fn set_hidden_textarea_with_selection(&self) {
         if let Some(selected_text) = self.text_buffer.selected_text() {
             if let Some(ref hidden_textarea) = self.hidden_textarea {
-                log::trace!("setting the value to textarea: {}", selected_text);
                 hidden_textarea.set_value(&selected_text);
-                log::trace!("textarea value: {}", hidden_textarea.value());
                 hidden_textarea.select();
             }
         }
     }
 
+    /// clear the text selection
+    pub fn clear_selection(&mut self) {
+        self.text_buffer.clear_selection();
+    }
+
     /// this is for newer browsers
     /// This doesn't work on webkit2
     #[cfg(web_sys_unstable_apis)]
@@ -486,9 +559,7 @@ impl<XMSG> Editor<XMSG> {
 
         if let Some(selected_text) = self.text_buffer.selected_text() {
             if let Some(ref hidden_textarea) = self.hidden_textarea {
-                log::trace!("setting the value to textarea: {}", selected_text);
                 hidden_textarea.set_value(&selected_text);
-                log::trace!("textarea value: {}", hidden_textarea.value());
                 hidden_textarea.select();
                 let html_document: HtmlDocument =
                     sauron::document().unchecked_into();
@@ -529,31 +600,29 @@ impl<XMSG> Editor<XMSG> {
             "Tab" => {
                 log::trace!("tab key is pressed");
                 let tab = "    ";
-                self.text_buffer.command_insert_text(tab);
+                self.command_insert_text(tab);
                 self.refocus_hidden_textarea();
             }
             "Enter" => {
-                self.text_buffer.command_break_line();
+                self.command_break_line();
             }
             "Backspace" => {
-                let c = self.text_buffer.command_delete_back();
-                self.recorded.delete(c);
+                self.command_delete_back();
             }
             "Delete" => {
-                let c = self.text_buffer.command_delete_forward();
-                self.recorded.delete_forward(c);
+                self.command_delete_forward();
             }
             "ArrowUp" => {
-                self.text_buffer.move_up();
+                self.command_move_up();
             }
             "ArrowDown" => {
-                self.text_buffer.move_down();
+                self.command_move_down();
             }
             "ArrowLeft" => {
-                self.text_buffer.move_left();
+                self.command_move_left();
             }
             "ArrowRight" => {
-                self.text_buffer.move_right();
+                self.command_move_right();
             }
             _ => (),
         }
