@@ -82,11 +82,17 @@ impl TextBuffer {
         self.selection_end = Some(end);
     }
 
+    /// clear the text selection
     pub fn clear_selection(&mut self) {
         self.selection_start = None;
         self.selection_end = None;
     }
 
+    pub fn select_all(&mut self) {
+        self.selection_start = Some(Point2::new(0, 0));
+        self.selection_end = Some(self.max_position());
+    }
+
     /// return the min and max selection bound
     pub fn normalize_selection(
         &self,
@@ -199,6 +205,7 @@ impl TextBuffer {
         start: Point2<usize>,
         end: Point2<usize>,
     ) -> String {
+        log::trace!("cutting from {} to {}", start, end);
         let deleted_text = self.get_text(start, end);
         if self.options.use_block_mode {
             for line_index in start.y..=end.y {
@@ -211,11 +218,12 @@ impl TextBuffer {
             if is_one_line {
                 self.lines[start.y].delete_cells(start.x, end.x);
             } else {
+                // at the last line of selection: delete the cells from 0 to end.x
+                self.lines[end.y].delete_cells_from_start(end.x);
+                // drain the lines in between
                 self.lines.drain(start.y + 1..end.y);
                 // at the first line of selection: delete the cells from start.x to end
                 self.lines[start.y].delete_cells_to_end(start.x);
-                // at the last line of selection: delete the cells from 0 to end.x
-                self.lines[end.y].delete_cells_from_start(end.x);
             }
         }
         deleted_text
@@ -665,6 +673,13 @@ impl TextBuffer {
         }
     }
 
+    pub(crate) fn join_line(&mut self, x: usize, y: usize) {
+        if self.lines.get(y + 1).is_some() {
+            let next_line = self.lines.remove(y + 1);
+            self.lines[y].push_ranges(next_line.ranges);
+        }
+    }
+
     fn assert_chars(&self, ch: char) {
         assert!(
             ch != '\n',
@@ -768,10 +783,38 @@ impl TextBuffer {
         self.lines.insert(line_index, line);
     }
 
+    /// return the line where the cursor is located
+    fn focused_line(&self) -> Option<&Line> {
+        self.lines.get(self.cursor.y)
+    }
+
+    /// return the position of the cursor
     pub(crate) fn get_position(&self) -> Point2<usize> {
         self.cursor
     }
 
+    /// the last line and last char of the text buffer
+    fn max_position(&self) -> Point2<usize> {
+        let last_y = self.lines.len().saturating_sub(1);
+
+        // if in block mode use the longest line
+        let last_x = if self.options.use_block_mode {
+            self.lines
+                .iter()
+                .map(|line| line.width.saturating_sub(1))
+                .max()
+                .unwrap_or(0)
+        } else {
+            // else use the width of the last line
+            if let Some(last_line) = self.lines.get(last_y) {
+                last_line.width.saturating_sub(1)
+            } else {
+                0
+            }
+        };
+        Point2::new(last_x, last_y)
+    }
+
     fn calculate_offset(&self, text: &str) -> (usize, usize) {
         let lines: Vec<&str> = text.lines().collect();
         let cols = if let Some(last_line) = lines.last() {
@@ -822,10 +865,17 @@ impl TextBuffer {
         self.cursor.x = 0;
         self.calculate_focused_cell();
     }
+
     pub(crate) fn move_right(&mut self) {
         self.cursor.x = self.cursor.x.saturating_add(1);
         self.calculate_focused_cell();
     }
+    pub(crate) fn move_right_end(&mut self) {
+        let line_width = self.focused_line().map(|l| l.width).unwrap_or(0);
+        self.cursor.x += line_width;
+        self.calculate_focused_cell();
+    }
+
     pub(crate) fn move_x(&mut self, x: usize) {
         self.cursor.x = self.cursor.x.saturating_add(x);
         self.calculate_focused_cell();
@@ -847,11 +897,19 @@ impl TextBuffer {
         self.cursor.y = y;
         self.calculate_focused_cell();
     }
-    pub(crate) fn command_break_line(&mut self) {
-        self.break_line(self.cursor.x, self.cursor.y);
+    pub(crate) fn command_break_line(&mut self, x: usize, y: usize) {
+        self.break_line(x, y);
         self.move_left_start();
         self.move_down();
+        self.calculate_focused_cell();
     }
+
+    pub(crate) fn command_join_line(&mut self, x: usize, y: usize) {
+        self.join_line(x, y);
+        self.set_position(x, y);
+        self.calculate_focused_cell();
+    }
+
     pub(crate) fn command_delete_back(&mut self) -> Option<char> {
         if self.cursor.x > 0 {
             let c = self
